#!/bin/sh

addfile() {
  cp -af $1 $ROOTPATH/$1
}

addbin() {
  n=0
  for file in "$@"; do
    if [ -f $file ]; then
      n=`expr $n + 1`
      cp -f $file $ROOTPATH/$file
      if ldd $file > /dev/null; then
        for i in `ldd $file | awk '/=> *\(/ {next} {print $3}' \
            | sed -e 's/^\/lib\/tls/\/lib/g' -e 's/^\/lib\/i[456]86/\/lib/g'`
        do
          addbin $i
        done
      fi
    fi
  done
  if [ "$n" = "0" ]; then
    echo "Warning: Commands $@ are missing."
  fi
}

adduser() {
  grep ^$1 /etc/passwd >> $ROOTPATH/etc/passwd
}

addgroup() {
  grep ^$1 /etc/group | sed -e 's/:[^:]*$/:/g' >> $ROOTPATH/etc/group
}

if [ -d "$1" ]; then
  export ROOTPATH="$1"
  shift 1
else
  echo "Usage:"
  echo "  `basename $0` chroot_directory [--auto] [--clamav] [--bogofilter] [--qsf]"
  echo "  chroot_directory must exist"
  echo "Where:"
  echo "  --auto       - select all options automatically"
  echo "  --clamav     - install clamav"
  echo "  --nod        - install nod32"
  echo "  --avg        - install avglinux"
  echo "  --bogofilter - install bogofilter"
  echo "  --qsf        - install quick spam filter"
  exit
fi

for i in dev etc bin sbin lib tmp/quarantine usr/lib usr/bin usr/sbin usr/share var/log/sagator var/lib; do
  mkdir -p "$ROOTPATH"/$i
done
ln -sf "$ROOTPATH/var/log/sagator" /var/log/

cat > "$ROOTPATH/etc/nsswitch.conf" <<EOF
passwd:     files
shadow:     files
group:      files

hosts:      files,dns

ethers:     files
netmasks:   files
networks:   files
protocols:  files
rpc:        files
services:   files
EOF

cat > "$ROOTPATH/etc/hosts" <<EOF
127.0.0.1	localhost.localdomain localhost
EOF

rm -f "$ROOTPATH/etc/passwd"
adduser root
adduser vscan

rm -f "$ROOTPATH/etc/group"
addgroup root
addgroup vscan

unset CLAMAV NOD AVG BOGOFILTER QSF
for arg in "$@"; do
  if [ "$arg" = "--auto" ]; then
    if [ -x /usr/bin/clamscan ]; then
      CLAMAV="YES"
    fi
    if [ -x /opt/nod32/nod32d -o -x /usr/local/nod32/nod32d ]; then
      NOD="YES"
    fi
    if [ -x /usr/local/bin/avgscan ]; then
      AVG="YES"
    fi
    if [ -x /usr/bin/bogofilter ]; then
      BOGOFILTER="YES"
    fi
    if [ -x /usr/bin/qsf ]; then
      QSF="YES"
    fi
  elif [ "$arg" = "--clamav" ]; then
    CLAMAV="YES"
  elif [ "$arg" = "--nod" ]; then
    NOD="YES"
  elif [ "$arg" = "--avg" ]; then
    AVG="YES"
  elif [ "$arg" = "--bogofilter" ]; then
    BOGOFILTER="YES"
  elif [ "$arg" = "--qsf" ]; then
    QSF="YES"
  else
    addbin "$arg"
  fi
done

if [ "$CLAMAV" ]; then
  adduser clamav
  addgroup clamav
  addbin /usr/bin/clam*scan
  addbin /usr/sbin/clamd
  if [ -f "$ROOTPATH/etc/clamav.conf" ]; then
    echo "Warning: /etc/clamav.conf found in $ROOTPATH, update skipped!"
  else
    addfile /etc/clamav.conf
  fi
  mkdir -p "$ROOTPATH/var/lib/clamav" "$ROOTPATH/var/log/clamav" \
    "$ROOTPATH/var/run/clamav"
  cp -f /var/lib/clamav/* "$ROOTPATH/var/lib/clamav"
  echo 'Run: su vscan -c "/usr/bin/freshclam --datadir='$ROOTPATH'/var/lib/clamav"'
  [ "`id -u`" = "0" ] && chown -R clamav.vscan "$ROOTPATH/var/lib/clamav" \
    "$ROOTPATH/var/run/clamav" "$ROOTPATH/var/log/clamav"
  chmod 770 "$ROOTPATH/var/run/clamav"
fi
if [ "$NOD" ]; then
  for nodpath in /opt/nod32 /usr/local/nod32; do
    if [ -x $nodpath/nod32d ]; then
      echo "Warning: Please reinstall NOD32 in $ROOTPATH$nodpath !"
    fi
  done
fi
if [ "$AVG" ]; then
  mkdir -p "$ROOTPATH/usr/local/bin" "$ROOTPATH/usr/local/lib/avg7"
  if [ -f "$ROOTPATH/etc/avg.conf" ]; then
    echo "Warning: /etc/avg.conf found in $ROOTPATH, update skipped!"
  else
    addfile /etc/avg.conf
  fi
  addbin /usr/local/lib/avg7/avgscan
  addbin /usr/local/lib/avg7/avgupdate
  cp -f /usr/local/lib/avg7/*.* "$ROOTPATH/usr/local/lib/avg7/"
  chown -R vscan.vscan "$ROOTPATH/usr/local/lib/avg7"
  echo "Warning: Do not forget to update AVG database in $ROOTPATH !"
fi
if [ "$BOGOFILTER" ]; then
  addbin /usr/bin/bogofilter
  addfile /etc/bogofilter.cf
  echo "bogofilter_dir=/usr/share/bogofilter" >> "$ROOTPATH/etc/bogofilter.cf"
  mkdir -p "$ROOTPATH/usr/share/bogofilter"
  echo "Run: chroot $ROOTPATH /usr/bin/bogofilter -s SPAM -n NOSPAM"
fi
if [ "$QSF" ]; then
  addbin /usr/bin/qsf
  echo "Run: Update for QuickSpamFilter database"
fi

echo "/usr/lib" > "$ROOTPATH/etc/ld.so.conf"

# make required devices
rm -f "$ROOTPATH/dev/null" "$ROOTPATH/dev/zero" "$ROOTPATH/dev/urandom"
mknod "$ROOTPATH/dev/null" c 1 3
chmod 0666 "$ROOTPATH/dev/null"
mknod "$ROOTPATH/dev/zero" c 1 5
chmod 0666 "$ROOTPATH/dev/zero"
mknod "$ROOTPATH/dev/urandom" c 1 9
chmod 0444 "$ROOTPATH/dev/urandom"

addbin /lib/libnss_files.so.2

# add archivers
addbin /bin/gzip /usr/bin/gzip
addbin /bin/bzip2 /usr/bin/bzip2
addbin /bin/tar /usr/bin/tar
addbin /usr/bin/compress /bin/compress
for i in unzip lha arc; do
  [ -x /usr/bin/$i ] && addbin /usr/bin/$i
done
for i in rar arj zoo ace arc; do
  addbin /usr/bin/un$i /usr/bin/$i
done

ldconfig -r "$ROOTPATH"

chmod 1777 "$ROOTPATH/tmp"
chmod 770 "$ROOTPATH/tmp/quarantine"
if [ "`id -u`" = "0" ]; then
  chown -R vscan.vscan "$ROOTPATH/tmp" "$ROOTPATH/var/log/sagator"
fi
